# -*- coding: utf-8 -*-

# This module contains no utilities. No matter what you are looking
# for, it is not here.

# flake8: noqa

SENTENCES = [
    'A metafísica é uma consequência de estar mal disposto.',
    'Se os tubarões fossem homens, tratariam melhor dos peixes pequenos.',
    'Eu verifico que não tenho par nisto tudo neste mundo.',
    'O ódio contra a baixeza também endurece os rostos.',
    'Ah, que maçada quererem que eu seja de companhia!',
    'Vivemos num tempo de chantagem universal',
    'Queria casar, conseguiu uma submulher. Tiveram subfilhos. Subnutridos.',
    ' A gente se acostuma para poupar a vida.',
    'No dia em que a Muralha da China ficou pronta, para onde foram os pedreiros?'
]


LOGOS = [
    [
        ('bold_blue', ' _____         _\n'),
        ('bold_blue', '|_   _|       (_)\n'),
        ('bold_blue', '  | | ___'), ('bold_yellow', '__  __'), ('bold_blue',
                                                                '_  ___ '), ('bold_red', ' ___      '), ('bold_white', '_'), ('bold_red', ' _    _\n'),
        ('bold_blue', '  | |/ _ '), ('bold_yellow', '\ \/ /'), ('bold_blue',
                                                                ' |/ __|'), ('bold_red', '| _ )_  _'), ('bold_white', '(_)'), ('bold_red', ' |__| |\n'),
        ('bold_blue', '  | | (_)'), ('bold_yellow', ' >  <'), ('bold_blue',
                                                               '| | (__ '), ('bold_red', '| _ \ || '), ('bold_white', '| |'), ('bold_red', ' / _` |\n'),
        ('bold_blue', '  \_/\___'), ('bold_yellow', '/_/\_\\'), ('bold_blue',
                                                                 '_|\___|'), ('bold_red', '|___/\_,_'), ('bold_white', '|_|'), ('bold_red', '_\__,_|\n')
    ],

    [
        ('bold_blue', '  __\n'),
        ('bold_blue',  ' /|            /\n'),
        ('bold_blue',
         '( |  ___         ___'), ('bold_red', '  __              ___\n'),
        ('bold_blue', '  | |   )'), ('bold_yellow', '(_/_'), ('bold_blue', ' | |'), ('bold_red',
                                                                                     '    |__] |  |'), ('bold_white', ' |'), ('bold_red', ' |    |  \ \n'),
        ('bold_blue', '  | |__/ '), ('bold_yellow', ' / /'), ('bold_blue', ' | |__'), ('bold_red',
                                                                                       '  |__] |__|'), ('bold_white', ' |'), ('bold_red', ' |___ |__/\n'),

    ],

    [
        ('bold_blue', '___________          .__        '), ('bold_red',
                                                            "'||                "), ('bold_white', '||'), ('bold_red', "  '||       '||\n"),
        ('bold_blue', '\__    ___/___'), ('bold_yellow', '___  __'), ('bold_blue', '|__| ____   '), ('bold_red',
                                                                                                     "|| ...  ... ...  "), ('bold_white', "..."), ("bold_red", "   ||     .. ||\n"),
        ('bold_blue', '  |    | /  _ '), ('bold_yellow', '\  \/  /'), ('bold_blue', '  |/ ___\  '), ('bold_red',
                                                                                                     "||'  ||  ||  ||   "), ('bold_white', "||"), ('bold_red', "   ||   .'  '||\n"),
        ('bold_blue', '  |    |(  <_> '), ('bold_yellow', '>    <'), ('bold_blue', '|  \  \___  '), (
            'bold_red', "||    |  ||  ||   "), ('bold_white', "||"), ('bold_red', "   ||   |.   ||\n"),
        ('bold_blue', '  |____| \____'), ('bold_yellow', '/__/\_ \\'), ('bold_blue', '__|\___  > '), ('bold_red',
                                                                                                      "'|...'   '|..'|. "), ('bold_white', ".||."), ('bold_red', " .||.  '|..'||.\n"),
        ('bold_blue', '                    '), ('bold_yellow',
                                                '\/'), ('bold_blue', '       \/')

    ]
]
