<div class="modal fade" id="outputPluginModal" tabindex="-1" role="dialog" aria-labelledby="Output Methods">
  <div class="modal-dialog" role="document">
    <div class="modal-content">

      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <h4 class="modal-title" id="outputPluginModalLabel">Output Methods</h4>
      </div>

      <div class="modal-body">
	<div id="plugins-container"></div>
      </div>
	<!-- modal footer -->
	<div class="modal-footer">
          <button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
          <button type="submit" class="btn btn-primary" id="btn-save-plugins">Save</button>
	</div>
      </form>
    </div>
  </div>
</div>
