<div class="modal fade" id="startBuildModal" tabindex="-1" role="dialog" aria-labelledby="StartBuild">
  <div class="modal-dialog" role="document">
    <div class="modal-content">

      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <h4 class="modal-title" id="startBuildModalLabel">Start Build</h4>
      </div>

      <!-- modal body -->
      <form role="form" id="start-build-form">
	<div class="modal-body">

	    <input type="hidden" name="start_build_name" id="start_build_name" class="form-control"/><br/>

	  <div class="form-group">
	    <label for="branch" class="control-label">Branch:</label>
	    <input type="text" name="branch" id="branch" class="form-control" required /><br/>
	  </div>

	  <div class="form-group">
	    <label for="builder_name" class="control-label">Builder name:</label>
	    <input type="text" name="builder_name" id="builder_name" class="form-control" /><br/>
	  </div>

	  <div class="form-group">
	    <label for="named_tree" class="control-label">Named tree:</label>
	    <input type="text" name="named_tree" id="named_tree" class="form-control" /><br/>
	  </div>

	  <div class="form-group">
	    <label for="build_slaves" class="control-label">Slaves:</label>
	    <select multiple class="form-control" id="build_slaves">
	      {% for slave in slaves %}
	      <option value="{{slave.name}}">{{slave.name}}</option>
	      {% end %}
	    </select><br/>
	  </div>

	</div>

	<!-- modal footer -->
	<div class="modal-footer">
          <button type="submit" class="btn btn-primary">Start build</button>
	</div>
      </form>
    </div>
  </div>
</div>
