<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">

    <title>Toxicbuild</title>

    <!-- Bootstrap -->
    <link href="{{static_url}}toxicbuild/css/bootstrap.min.css" rel="stylesheet">
    <link href="{{static_url}}toxicbuild/css/font-awesome.min.css" rel="stylesheet">
    <link href="{{static_url}}toxicbuild/css/toxicbuild.css" rel="stylesheet">

  </head>
  <body>

    <nav class="navbar navbar-inverse">
      <div class="container toxicweb-container">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="/">
	    <img class="img-logo" src="{{static_url}}toxicbuild/img/Logo.svg"/>
	  </a>
        </div>
        <div id="navbar" class="collapse navbar-collapse">
          <ul class="nav navbar-nav">
            {% block master_msg %}
	    <li class="master-msg">Connected to master at {{master_location}} | version: {{version}}</li>
	    {% end %}
            <!-- <li><a href="#about">About</a></li> -->
            <!-- <li><a href="#contact">Contact</a></li> -->
          </ul>
        </div><!--/.nav-collapse -->
      </div>
      {% block logout_link %}
      <div class="logout-link-container">
	<a href="/logout"><span class="glyphicon glyphicon-log-out" aria-hidden="true"></span> Logout</a>
      </div>
      {% end %}
    </nav>

    <div class="container-fluid">
	<!-- Error alert -->
	<div class="alert alert-danger alert-dismissible" role="alert">
	  <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
	  <div id="error-container"></div>
	</div>

	<!-- Success alert -->
	<div class="alert alert-success alert-dismissible" role="alert">
	  <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
	  <div id="success-container"></div>
	</div>

      <div class="jumbotron-fluid background-main">

	{% block main_area %}{% end %}
      </div>
    </div>
    {% block js_bottom%}{% end %}
  </body>
</html>
