/* base */
.navbar{
    margin-bottom: 0;
}
.jumbotron{
    padding-top:0;
}

/* alert */
.alert{
    position: fixed;
    width: 97.2%;
    margin-top: -35px;
    z-index: 1000;
    opacity: 0.98;
    box-shadow: 0px 0px 5px 1px #000;
}

/* main */
.background-main,
body{
    background-color: #4c4a49;
    color: #D0D0D0;
}

.page-header{
    magin: 0;
    margin-top: 20px;
}

a:hover,
a {
    color: #1A8FF5;
    text-decoration: none;
}


.table>thead>tr>th {
    border-bottom: 0;
}

tr:nth-child(even) {
    background-color: #696565;
}

.main-row{
    margin-left: 0;
    margin-right: 0;
}

.main-section-title{
    float:left;
}

.modal-content{
    background-color: #4c4a49;
}

.repo-name-group .left-container{
    float: left;
    width: 82%;
    padding-right: 10px;
}

.repo-name-group .right-container{
    width: 18%;
    float: left;
}

.form-group-vcs-type-update{
    min-height: 80px;
}

.form-group-vcs-type-update .left-container{
    float:left;
    width: 65%;
    margin-right: 5%;;
}

.form-group-vcs-type-update .right-container{
    width: 30%;
    float: left;
}

.add-branch-icon{
    color: #49b149;
    font-weight: bolder;
    cursor: pointer;
}

#branches-container{
    padding: 10px;
}

.branch-name-input{
    float: left;
    width: 69%;
    margin-right: 4%;
}

.remove-branch{
    color: #da4242;
    margin-left: 80%;
    cursor: pointer;
}

.form-group-slave-host-port .left-container{
    float:left;
    width: 65%;
    margin-right: 5%;;
}

.form-group-slave-host-port .right-container{
    width: 30%;
    float: left;
}

.page-header{
    border: 0;
}
.toxicweb-container{
    margin-left: 0;
    float:left;
    width: 1000px;
}

.logout-link-container{
    padding-top: 22px;
    float: right;
    margin-right: 1%;
}

.alert{
    display:none;
}

.img-logo{
    width: 190px;
    margin-top: -4%;
}


.master-msg{
    color: #fff;
    margin-top:9%;
}


.btn-transparent,
.btn-transparent:focus,
.btn-transparent:hover,
.btn-transparent:active,
.btn-transparent.active:focus,
.btn-transparent:visited,
.btn-transparent.active:focus,
.btn-transparent:active:focus,
.btn-rebuild,
.btn-rebuild:focus,
.btn-rebuild:hover,
.btn-rebuild:active,
.btn-rebuild.active:focus,
.btn-rebuild:visited,
.btn-rebuild.active:focus,
.btn-rebuild:active:focus,
.btn-main-edit:focus,
.btn-main-edit:hover,
.btn-main-edit.active:focus,
.btn-main-edit:active,
.btn-main-edit:visited,
.btn-main-edit.active:focus,
.btn-main-edit:active:focus,
.btn-main-edit{
    background-color: transparent;
    border-color: transparent;
    outline: none;
}

.btn-transparent{
    color: #000;
}

.btn-transparent:hover,
.btn-transparent.active:focus,
.btn-rebuild:hover,
.btn-rebuild.active:focus,
.btn-main-edit:hover,
.btn-main-edit.active:focus,
.btn-main-edit:active{
    color: #b5a5a5;
}

.btn-main-add:focus,
.btn-main-add:hover,
.btn-main-add:active,
.btn-main-add:visited,
.btn-main-add.active:focus,
.btn-main-add:active:focus,
.btn-main-add{
    background-color: transparent;
    border-color: transparent;
    outline: none;
    color: #49b149;
    margin-top: 8px;
    margin-left: 4px;
}

.btn-status{
    width: 80px;
}

.toxic-spinner-running,
.toxic-spinner-main{
    transform: translateZ(0);
    -webkit-transform: translateZ(0);
    -ms-transform: translateZ(0);
    background-image: url("/static/toxicbuild/img/cog-dark-bg.gif");
    width: 17px;
    height: 17px;
    font-size: 1.11em;
}

tr:nth-child(even) .toxic-spinner-main{
    background-image: url("/static/toxicbuild/img/cog-light-grey-bg.gif");
}

.toxic-spinner-running{
    background-image: url("/static/toxicbuild/img/cog-blue-bg.gif");
    position: absolute;
    margin-top: 0.1%;
    margin-left: 0.5%;
}

.repo-status-form{
    float: left;
}

.btn-toxicbuild-default {
  color: #fff;
  background-color: #868B90;
  border-color: #7A7E83;
}


.btn-toxicbuild-default:hover {
  color: #fff;
  background-color: #575C61;
  border-color: #7A7E83;
}

#btn-delete-repo{
    display: none;
}

#btn-delete-slave{
    display: none;
}

.plugin-name img{
    width: 20px;
    height: 20px;
}

.enable-plugin-input{
    float: left;
    width: 15px;
    height: 15px;
    margin-top: -13%;
}

.enable-plugin-label{
    float: left;
    margin-left: -20%;
    margin-right: 15%;
}

.enable-plugin-container{
    float: right;
}

.plugin-container{
    margin-bottom: 2%;
}

/* waterfall */

.build-details-btn{
    float:right;
    margin-right: -3px;
}
.step{
    border-bottom: 1px solid #4c4a49;
}

.step a{
    color: #05335A;
}

.toxic-ul{
    list-style: none;
}

.builder-stick{
    position: fixed;
    top: 0;
    z-index:999;
}

.buildsets-column{
    max-width: 160px;
}

.builder-column{
    vertical-align: top;
}

.builder-container{
    border-radius: 3px;
    min-height: 50px;
}

.builder{
    box-shadow: 0px 1px 5px 0px #000;
}

.builder,
.step{
    border-radius: 3px;
    list-style: none;
    padding: 10px
}

.builder-fail,
.step-fail{
    background-color: #d9534f;

}

.build-step-info-container{
    overflow: hidden;
}

.step-info-text{
    float:left;
}
.step-info-btn{
    float: right;
}

.step-info-text{
    max-width: 85%;
}


.builder-success,
.step-success{
    background-color: #3caf3c;
}

.btn-pending,
.builder-pending,
.step-pending{
    background-color: #B7B206;
}

.builder-running,
.step-running{
    background-color: #31b0d5;
}

.builder-warning,
.step-warning{
    background-color: #d8672f;
}

.btn-exception,
.builder-exception,
.step-exception{
    background-color: #75094A;
}

.btn-step-details:focus,
.btn-step-details{
    padding: 1px 6px;
    float: right;
}

.btn-step-details:hover{
    padding: 1px 6px;
    float: right;
}

.btn-buildset-details:focus,
.btn-buildset-details,
.btn-buildset-details:hover{
    margin: 0;
    float: left;
}

.modal-dialog-step{
    width:90%;
}

#waterfall-table tr:nth-child(even) {
    background-color: #4c4a49;
}

.buildset{
    list-style: none;
}

/* login and logout form */

.form-signin {
  max-width: 330px;
  padding: 15px;
  margin: 0 auto;
  margin-top: 25px;
}
.login-input-container{
    margin-top: 20px;
}

.form-signin .form-signin-heading,
.form-signin .checkbox {
  margin-bottom: 10px;
}
.form-signin .checkbox {
  font-weight: normal;
}
.form-signin .form-control {
  position: relative;
  height: auto;
  -webkit-box-sizing: border-box;
     -moz-box-sizing: border-box;
          box-sizing: border-box;
  padding: 10px;
  font-size: 16px;
}
.form-signin .form-control:focus {
  z-index: 2;
}
.form-signin input[type="email"] {
  margin-bottom: -1px;
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
}
.form-signin input[type="password"] {
  margin-bottom: 10px;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.alert-invalid-credentials{
    display: block;
    width: 23%;
}
