<div class="modal fade" id="stepDetailsModal" tabindex="-1" role="dialog" aria-labelledby="Show step">
  <div class="modal-dialog modal-dialog-step" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <h4 class="modal-title" id="stepDetailsModalLabel">Step details</h4>
      </div>

      <!-- modal body -->
      <div class="modal-body">
	<ul class="toxic-ul">
	  <li>command: <span id="step-command"></span></li>
	  <li>status: <span id="step-status"></span></li>
	  <li>started: <span id="step-start"></span></li>
	  <li>finished: <span id="step-end"></span></li>
	  <li>total: <span id="step-total-time"></span></li>
	  <li>output: <span><a href="#" id="follow-step-output">Follow output</a></span>
	    <pre id="step-output"></pre>
	  </li>
	</ul>
      </div>

	<!-- modal footer -->
	<div class="modal-footer">
	  <button type="button" class="btn btn-default" data-dismiss="modal">Close</button>
	</div>
    </div>
  </div>
</div>
