# -*- coding: utf-8 -*-


class BuilderNotFound(Exception):
    pass


class BadData(Exception):
    pass


class BadBuilderConfig(Exception):
    pass


class BusyRepository(Exception):
    pass


class NotConnected(Exception):
    pass
